# coding: utf-8
from django.test import mock
from django.contrib.auth.models import User
from django.conf import settings
from rest_framework import test, status


class AuthHelperMixin(object):

    def setUp(self):
        super(AuthHelperMixin, self).setUp()
        self.requests_patcher = mock.patch('sw_rest_auth.permissions.requests')
        self.requests_mock = self.requests_patcher.start()

    def tearDown(self):
        super(AuthHelperMixin, self).tearDown()
        self.requests_mock.stop()

    def assertPermChecked(self, user, perm):
        self.assertTrue(self.requests_mock.get.called)
        call_args, call_kwargs = self.requests_mock.get.call_args
        self.assertEqual(settings.AUTH_SERVICE_CHECK_PERM_URL, call_args[0])
        self.assertEqual(user.username, call_kwargs['params']['user'])
        self.assertEqual(perm, call_kwargs['params']['perm'])

    def force_permission(self, user=None, perm=None, perm_list=None):
        if not perm_list and perm:
            perm_list = [perm]

        def side_effect(url, headers, params):
            response_mock = mock.Mock()
            if params['user'] == user.username and params['perm'] in perm_list:
                response_mock.status_code = status.HTTP_200_OK
            else:
                response_mock.status_code = status.HTTP_400_BAD_REQUEST
            return response_mock

        if user and perm_list:
            self.requests_mock.get.side_effect = side_effect
        else:
            self.requests_mock.get.side_effect = None


class AuthTestCaseMixin(AuthHelperMixin):
    url = None
    perm = None
    method_name = 'post'

    def get_method_names(self):
        if self.get_perm_map():
            return self.get_perm_map().keys()
        else:
            return [self.method_name]

    def get_perm_map(self):
        if self.perm:
            return {self.method_name: self.perm}
        else:
            return None

    def setUp(self):
        super(AuthTestCaseMixin, self).setUp()
        self.client.force_authenticate(self.get_user())
        if self.get_perm_map():
            self.force_permission(self.get_user(), perm_list=self.get_perm_map().values())

    def test_not_auth(self):
        self.client.force_authenticate()         # unset authentication
        for method_name in self.get_method_names():
            response = getattr(self.client, method_name)(self.url)
            self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_forbidden(self):
        if not self.get_perm_map():
            return

        self.client.force_authenticate(user=self.get_user())
        self.force_permission()     # unset permission
        for method_name, perm_code in self.get_perm_map().items():
            response = getattr(self.client, method_name)(self.url)
            self.assertPermChecked(self.get_user(), perm_code)
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_perm(self):
        if not self.get_perm_map():
            return

        self.client.force_authenticate(self.get_user())
        for method_name, perm_code in self.get_perm_map().items():
            self.force_permission(self.get_user(), perm=perm_code)
            response = getattr(self.client, method_name)(self.url)
            self.assertPermChecked(self.get_user(), perm_code)
            self.assertNotEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
            self.assertNotEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def get_user(self):
        if not hasattr(self, 'user') or not self.user:
            self.user = User.objects.create(username='tester')
        return self.user




# coding: utf-8
from django.conf.urls import url
from . import views

urlpatterns = [
    url(r'^check_token/$', views.check_token),
    url(r'^check_login_password/$', views.check_login_password),
    url(r'^check_perm/$', views.check_perm),
    url(r'^api_token_auth/$', views.obtain_auth_token),
]



# coding: utf-8
import requests
from django.conf import settings
from django.contrib.auth.models import User


class RestBackend(object):
    def authenticate(self, username, password):
        url = settings.AUTH_SERVICE_CHECK_LOGIN_PASSWORD_URL
        auth_token = settings.AUTH_TOKEN
        auth_verified_ssl_crt = getattr(settings, 'AUTH_VERIFIED_SSL_CRT_PATH', None)

        headers = {'Authorization': 'Token %s' % auth_token}
        data = {'username': username, 'password': password}
        try:
            kwargs = {
                'headers': headers,
                'data': data,
            }
            kwargs['verify'] = auth_verified_ssl_crt
            r = requests.post(url, **kwargs)
        except requests.ConnectionError:
            return None

        if r.status_code == 200:
            result = r.json()
            result.pop('id')
            result.pop('user_permissions')
            result.pop('groups')
            try:
                user = User.objects.get(username=result['username'])
            except User.DoesNotExist:
                result['password'] = password
                user = User.objects.create_user(**result)
            return user

        return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None

    def has_perm(self, user, perm, obj):
        from .permissions import CodePermission
        return CodePermission.has_permission_by_params(user.username, perm, raise_exception=False)



# coding: utf-8
import logging
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
import rest_framework.authtoken.views
from . import serializers

logger = logging.getLogger('sw.rest.auth')


@api_view(['POST'])
@permission_classes((IsAuthenticated,))
def check_token(request):
    """
    Resource check is token valid.
    ---
    request_serializer: serializers.CheckToken
    type:
        username:
            required: true
            type: string
            description: token related user

    responseMessages:
        - code: 200
          message: Token is valid

        - code: 400
          message: Token is not valid

        - code: 401
          message: Unauthorized
    """
    serializer = serializers.CheckToken(data=request.data)
    serializer.is_valid(raise_exception=True)
    token = serializer.validated_data['token']
    logger.debug('Token correct', extra={'token': token, 'username': token.user.username})
    return Response({'username': token.user.username})


@api_view(['POST'])
@permission_classes((IsAuthenticated,))
def check_login_password(request):
    """
    Resource check login and password combination is valid.
    ---
    request_serializer: serializers.CheckLoginPassword
    response_serializer: serializers.User

    responseMessages:
        - code: 200
          message: Login and password is valid

        - code: 400
          message: Login and password is not valid

        - code: 401
          message: Unauthorized
    """
    serializer = serializers.CheckLoginPassword(data=request.data)
    serializer.is_valid(raise_exception=True)
    username = serializer.validated_data['username']
    logger.debug('Username and password correct', extra={'username': username})
    user_serializer = serializers.User(instance=serializers.AuthUser.objects.get(username=username))
    return Response(user_serializer.data)



@api_view(['GET'])
@permission_classes((IsAuthenticated,))
def check_perm(request):
    """
    Check user permission.
    ---
    request_serializer: serializers.CheckPerm

    parameters:
        - name: user
          type: string
          required: true
          description: username
          paramType: query

        - name: perm
          type: string
          required: true
          description: permission code
          paramType: query

    type:
        result:
            required: true
            type: string
            description: user have permission

    responseMessages:
        - code: 200
          message: User have permission

        - code: 400
          message: User have not permission

        - code: 401
          message: Unauthorized
    """
    serializer = serializers.CheckPerm(data=request.query_params)
    serializer.is_valid(raise_exception=True)
    logger.debug('User have perm', extra=serializer.data)
    return Response({'result': 'ok'})


@api_view(['POST'])
def obtain_auth_token(request):
    """
    Getting api-token by login and password.
    ---
    request_serializer: rest_framework.authtoken.serializers.AuthTokenSerializer
    type:
        token:
            required: true
            type: string
            description: User token for using in request header "Authentication"
    """
    return rest_framework.authtoken.views.obtain_auth_token(request)


# coding: utf-8
# python setup.py sdist register upload
from setuptools import setup

setup(
    name='sw-django-rest-auth',
    version='0.0.25',
    description='Soft Way company django restfromework authentication service package.',
    author='Telminov Sergey',
    url='https://github.com/telminov/sw-django-rest-auth',
    packages=[
        'sw_rest_auth',
        'sw_rest_auth/migrations',
        'sw_rest_auth/tests',
    ],
    include_package_data=True,
    license='The MIT License',
    test_suite='runtests.runtests',
    install_requires=[
        'django',
        'djangorestframework',
        'requests',
    ],
)